﻿using Exact.Identity;
using Exact.Identity.Enumerations;
using Exact.Services.Client;
using Exact.Services.Client.Data.Entities;
using Exact.Services.Client.Entities;
using Exact.Services.Client.Enumerations;
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace FIDWebServiceClient
{
    public partial class FIDWebServiceClient : Form
    {
        private string auth0Connection;
        private string authority;
        private string clientid;
        private string username;
        private string password;
        private string serviceUrl;
        private string entityName;
        private int batchSize;

        public FIDWebServiceClient()
        {
            InitializeComponent();
            txtAuthority.Text = "synergy.auth0.com";
            txtAuth0Connection.Text = "Auth0-ESE-SYS";
            txtClientID.Text = "NCaYfJIFQSmvKZvSsLghlCFVAUHFd56G";
            txtUsername.Text = "roney.tham@exact.com";
            txtPassword.Text = "Exact_1234";
            txtEntityName.Text = "ItemUnit";
            txtBatchSize.Text = "2";
            txtURL.Text = "http://localhost/000/services";
        }

        private void txtBatchSize_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsControl(e.KeyChar) && !char.IsDigit(e.KeyChar) && (e.KeyChar != '.'))
                e.Handled = true;

            // only allow one decimal point
            if ((e.KeyChar == '.') && ((sender as TextBox).Text.IndexOf('.') > -1))
                e.Handled = true;
        }

        private void btnRetrieve_Click(object sender, EventArgs e)
        {
            try
            {
                SetValues();
                var parameters = SetParameters();
                var securityToken = AcquireToken(parameters);
                var authenticationProvider = GetAuthenticationProvider(securityToken);
                var entitiesData = GetEntitiesService(authenticationProvider);
                WriteResult(entitiesData);
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void SetValues()
        {
            auth0Connection = txtAuth0Connection.Text;
            authority = txtAuthority.Text;
            clientid = txtClientID.Text;
            username = txtUsername.Text;
            password = txtPassword.Text;
            serviceUrl = txtURL.Text;
            entityName = txtEntityName.Text;
            batchSize = Convert.ToInt32(txtBatchSize.Text);
        }

        private List<KeyValuePair<KeyName, string>> SetParameters()
        {
            List<KeyValuePair<KeyName, string>> parameters = new List<KeyValuePair<KeyName, string>>() {
                new KeyValuePair<KeyName, string>(KeyName.Auth0Connection, auth0Connection),
                  new KeyValuePair<KeyName, string>(KeyName.Authority, authority),
                  new KeyValuePair<KeyName, string>(KeyName.ClientID, clientid),
                  new KeyValuePair<KeyName, string>(KeyName.Username, username),
                  new KeyValuePair<KeyName, string>(KeyName.Password, password)
                  };
            return parameters;
        }

        private TokenResult AcquireToken(List<KeyValuePair<KeyName, string>> parameters)
        {
            try
            {
                IdentityConnectionMetadata icMetadata = new IdentityConnectionMetadata(AuthenticationProtocol.OAuth, ConnectionMode.Active, IdentityProviderType.Auth0);
                icMetadata.Parameters = parameters;
                IdentityConnectionManager icManager = new IdentityConnectionManager(icMetadata);
                return icManager.AcquireAccessTokenExtended();
            }
            catch(Exception ex)
            {
                throw new Exception("Failed to acquire token. " + ex.Message);
            }
        }

        private AuthenticationProvider GetAuthenticationProvider(TokenResult securityToken)
        {
            WebSessionBehavior sessionObject = new WebSessionBehavior();
            AuthenticationProvider authProvider = new AuthenticationProvider()
            {
                Connection = serviceUrl,
                SecurityToken = securityToken.SecurityToken,
                SecurityTokenType = ClientAccessTokenType.OAUTH,
                Username = username,
                Password = password,
                SessionObject = sessionObject
            };
            return authProvider;
        }

        private EntitiesData GetEntitiesService(AuthenticationProvider authenticationProvider)
        {
            try
            {
                EntitiesClientWeb ent = ServiceCallFactory.CreateEntitiesClientWeb(AuthenticationMode.Federated, authenticationProvider);
                RetrieveCriteria rc = new RetrieveCriteria();
                rc.EntityName = entityName;
                rc.BatchSize = batchSize;
                var entitiesData = ent.RetrieveSet(rc);
                return entitiesData;
            }
            catch(Exception ex)
            {
                throw new Exception("Failed to retrieve service. " + ex.Message);
            }
        }

        private void WriteResult(EntitiesData entitiesData)
        {
            StringBuilder result = new StringBuilder();
            foreach(var data in entitiesData.Entities)
            {
                foreach (var prop in data.Properties)
                {
                    result.AppendLine(prop.Name + ": " + prop.Value);
                }
                result.AppendLine();
            }
            txtResult.Text = result.ToString();
        }
    }
}
